#include "validation.h"
#include <cmath>

// TODO: For INTERACTIVE PROBLEMS:
// Write your output validator as usual, but make sure to flush all standard
// output. Call `v.set_WA_handler(lambda)` to gracefully handle failures when
// parsing team output. You could e.g. send a '-1' to the submission to tell it
// to stop running.

// Interactor that sends requested elements of array in .in file and checks whether the solution is optimal
// TODO: Perhaps store this in a .ans file?

int maxq = 50;
int limitq = 500; // Used for debugging, run the program for limitq queries, but >maxq yields a wrong answer.

int main(int argc, char *argv[]) {
    // Set up the input and answer streams.
    std::ifstream in(argv[1]);
    //std::ifstream ans(argv[2]);
    OutputValidator v(argc, argv);

    // Read input data
    int n;
    in >> n;
    std::vector<int> arr;
    int a;
    for (int i = 0; i < n; i++)
    {
        in >> a;
        arr.push_back(a);
    }
    // Read possible optimal index
    //int optinx;
    //ans >> optinx;

    // Send initial length
    std::cout << n << std::endl;

    int usedqueries = 0;

    // Interact
    while(1)
    {
        std::string qrytype;
        std::vector<std::string> exp{"?", "!"};
        qrytype = v.test_strings(exp);
        v.space();
        if (qrytype == "?")
        {
            usedqueries++;
            if(usedqueries > limitq) v.WA("Used way too many queries");
            int inx;
            inx = v.read_integer("inx", 1, n);
            std::cout << arr[inx - 1] << std::endl;
        }
        else
        {
            if(usedqueries > maxq) v.WA("Used too many queries, " + std::to_string(usedqueries) + " instead of the maximum " + std::to_string(maxq));
            std::vector<long long int> inxs;
            inxs = v.read_integers("inxs", 3, 1, n);
            // Check whether values are unique
            if ((inxs[0] == inxs[1]) || (inxs[1] == inxs[2]) || (inxs[2] == inxs[0])) v.WA("Output contains two or more equal indices.");
            std::sort(inxs.begin(), inxs.end());
            // Check whether outer two are selected.
            if ((arr[0] != arr[inxs[0]-1]) || (arr[n-1] != arr[inxs[2]-1])) v.WA("One of outermost elements not selected");
            // Check if yields the same result as optmid
            //if ((arr[inxs[1]-1] != arr[optinx-1]) && (arr[inxs[1]-1] + arr[optinx - 1] != arr[0] + arr[n-1])) v.WA("Middle value not optimal");
            // Check if there is an index closer to (min + max) // 2
            int midval = arr[inxs[1]-1];
            if (2 * midval > arr[0] + arr[n-1]) midval = arr[0] + arr[n-1] - midval; //Gives equivalent score and now midval <= (arr[0] + arr[n-1]) // 2
            // Look for smallest value > midval
            if (midval == arr[n-1]) break; // Any answer is correct in this case.
            int lo = 0;
            int hi = n-1;
            while (hi - lo > 1)
            {
                int mid = (lo + hi) / 2;
                if (arr[mid] > midval) hi = mid;
                else lo = mid;
            }
            if (std::abs(2 * arr[hi] - (arr[0] + arr[n-1])) < std::abs(2 * midval - (arr[0] + arr[n-1])))
                v.WA("Non-optimal midpoint");
            break;
        }
    }
    std::cerr << "Queries used: " << usedqueries << std::endl;
}

