#include "validation.h"

using std::cerr;
using std::endl;
using std::to_string;
using std::vector;

// This output validator checks whether it's beneficial for anyone to switch
// contests. If anyone wants to switch the people with less skill also want to
// switch. Therefore, it suffices to just check if the person in each contest
// with the lowest skill wants to switch.

int main(int argc, char* argv[]) {
    // Set up the input and answer streams.
    std::ifstream in(argv[1]);
    // std::ifstream ans(argv[2]); // Only for custom checker.
    OutputValidator v(argc, argv);

    // Read the input file.
    long long n, m;
    in >> n >> m;

    vector<long long> skills = vector<long long>(n);
    for(auto& x : skills) {
        in >> x;
    }

    vector<long long> prizes = vector<long long>(m);
    for(auto& x : prizes) {
        in >> x;
    }

    // We keep track of all people in a contest and the total skill level of
    // each contest.
    vector<vector<long long>> contests;
    vector<long long> total_skill;

    for(long long i = 0; i < m; i++) {
        long long count = v.read_integer("count", 0, n);

        if(count > 0) {
            v.space();
            vector<long long> contest = v.read_integers("people", count, 1, n);

            // Make vector of people in a contest zero-based indexed and sorted
            // from high to low.
            for(long long& x : contest) {
                x--;
            }
            sort(contest.begin(), contest.end(),
                 [&](long long x, long long y) { return skills[x] > skills[y]; });

            contests.push_back(contest);

            // Compute the total skill level in each contest.
            long long temp = 0;
            for(long long x : contest) {
                temp += skills[x];
            }

            total_skill.push_back(temp);
        } else {
            contests.push_back(vector<long long>());
            total_skill.push_back(0);
            v.newline();
        }
    }

    // Try to assign the last person (with lowest skill) of each contest
    // to all other contests and see if it's better.
    for(long long i = 0; i < m; i++) {
        if(contests[i].size() == 0) {
            continue;
        }

        long long last       = contests[i].back();
        long long skill_last = skills[last];

        for(long long j = 0; j < m; j++) {
            if(i == j) {
                continue;
            }

            if(prizes[i] * (total_skill[j] + skill_last) < prizes[j] * total_skill[i]) {
                cerr << prizes[i] << " " << total_skill[i] << " " << prizes[j] << " "
                     << total_skill[j] << " " << skill_last << endl;
                v.WA("Switching between contest " + to_string(i + 1) + " and " + to_string(j + 1) +
                     " gives better expected value.");
            }
        }
    }
}