#!/usr/bin/env python3

from typing import List


def dfs(a: int, L: int, g: List[List[int]], btoa: List[int], A: List[int], B: List[int]) -> bool:
    if A[a] != L:
        return False
    A[a] = -1
    for b in g[a]:
        if B[b] == L + 1:
            B[b] = 0
            if btoa[b] == -1 or dfs(btoa[b], L + 1, g, btoa, A, B):
                btoa[b] = a
                return True
    return False


def hopcroft_karp(g: List[List[int]], btoa: List[int]) -> int:
    res = 0
    cur, next = [], []
    while True:
        A = [0] * len(g)
        B = [0] * len(btoa)
        cur.clear()
        for a in btoa:
            if a != -1:
                A[a] = -1
        for a in range(len(g)):
            if A[a] == 0:
                cur.append(a)
        for lay in range(1, int(1e15)):
            is_last = False
            next.clear()
            for a in cur:
                for b in g[a]:
                    if btoa[b] == -1:
                        B[b] = lay
                        is_last = True
                    elif btoa[b] != a and B[b] == 0:
                        B[b] = lay
                        next.append(btoa[b])
            if is_last:
                break
            if not next:
                return res
            for a in next:
                A[a] = lay
            cur, next = next, cur
        for a in range(len(g)):
            res += dfs(a, 0, g, btoa, A, B)


def permits_perfect_matching(g: List[List[int]], size_right: int) -> bool:
    btoa = [-1] * size_right
    return hopcroft_karp(g, btoa) == size_right


class Card:
    def __init__(self, letter, digit):
        self.letter = letter
        self.digit = digit

    def __iter__(self):
        return iter((self.letter, self.digit))


def print_vector(v):
    assert v
    print(len(v))
    print(' '.join(str(x + 1) for x in v))


n, k = map(int, input().split())
deck = [Card(*card) for card in input().split()]
table = input().split()
D, L = input().split()

legal, illegal = [], []
for i, (l, d) in enumerate(deck):
    if l != L and d == D:
        illegal.append(i)
    else:
        legal.append(i)

if not illegal:
    exit(print("true"))

graph = [[] for _ in range(n)]
for i in legal:
    l, d = deck[i]
    for j, c in enumerate(table):
        if l == c or d == c:
            graph[i].append(j)

if not permits_perfect_matching(graph, k):
    exit(print("false"))

check_letter = [False] * 26
for i in illegal:
    check_letter[ord(deck[i].letter) - ord('A')] = True
assert not check_letter[ord(L) - ord('A')]

flip = []
for j, c in enumerate(table):
    if c == D:
        flip.append(j)
    elif c >= 'A' and check_letter[ord(c) - ord('A')]:
        flip.append(j)

if not flip:
    print("true")
else:
    print_vector(flip)
