#include <iostream>
#include <queue>
#include <unordered_map>
#include <vector>

using namespace std;

int n, k, s, t;
int vertex_count;
unordered_map<int, int> v_by_key;
const int STRAIGHT = 0, LEFT = 1, RIGHT = 2;
const int DIRECTIONS[] = { STRAIGHT, LEFT, RIGHT };
const int BLINKING_DIRECTIONS[] = { LEFT, RIGHT };
vector<bool> incident_to_start;
vector<bool> incident_to_end;
vector<pair<int, int>> Es[3];
vector<vector<int>> adj;
vector<int> dist;

int create_vertex(int i, int j) {
    int key = i + n * j;
    auto v = v_by_key.find(key);
    if (v == v_by_key.end()) {
        incident_to_start.push_back(i == s);
        incident_to_end.push_back(j == t);
        return v_by_key[key] = vertex_count++;
    }
    return v->second;
}

void build_edge(vector<pair<int, int>> &E, int i, int j, int k) {
    if (j == 0 || k == 0)
        return;
    int v1 = create_vertex(j, i), v2 = create_vertex(i, k);
    E.emplace_back(v1, v2);
}

void link(int l1, int d1, int l2, int d2, vector<pair<int, int>> &E) {
    int offset1 = (3 * l1 + d1) * vertex_count;
    int offset2 = (3 * l2 + d2) * vertex_count;
    for (pair<int, int> e : E)
        adj[offset1 + e.first].push_back(offset2 + e.second);
}

int main() {

    // problem dimensions and start and end
    cin >> n >> k;
    s = 1, t = n;

    for (int i = 1; i <= n; ++i) { // note: 1 based indices, 0 indicates nonexistence
        int adj[4];
        for (int &j : adj) {
            cin >> j;
            if (i == s && j == t) { std::cout << 1 << std::endl; return 0; }  // band-aid fix
        }
        for (int l = 0; l < 4; ++l) {
            build_edge(Es[STRAIGHT], i, adj[l], adj[(l + 2) % 4]);
            build_edge(Es[LEFT],     i, adj[l], adj[(l + 1) % 4]);
            build_edge(Es[RIGHT],    i, adj[l], adj[(l + 3) % 4]);
        }
    }

    // edge case
    if (s == t) {
        cout << 0 << endl;
        return 0;
    }

    // construct layered graph
    adj = vector<vector<int>>(3 * (k + 1) * vertex_count);

    for (int l = 0; l <= k; ++l) {
        // within layer and direction transitions
        for (int d : DIRECTIONS)
            link(l, d, l, d, Es[d]);

        // may go from blinking to straight for free
        for (int d : BLINKING_DIRECTIONS)
            link(l, d, l, STRAIGHT, Es[STRAIGHT]);

        // may go from anything to blinking at the cost of one budget
        if (l < k)
            for (int d1 : DIRECTIONS)
                for (int d2 : BLINKING_DIRECTIONS)
                    if (d1 != d2)
                        link(l, d1, l + 1, d2, Es[d2]);
    }

    // search
    dist = vector<int>(adj.size(), -1);
    queue<int> Q;
    for (int i = 0; i < vertex_count; ++i) {
        if (incident_to_start[i]) {
            int v = (3 * 0 + STRAIGHT) * vertex_count + i;
            Q.push(v);
            dist[v] = 1;
        }
    }
    while (!Q.empty()) {
        int v = Q.front();
        Q.pop();
        for (int u : adj[v]) {
            if (dist[u] == -1) {
                Q.push(u);
                dist[u] = dist[v] + 1;
                if (incident_to_end[u % vertex_count]) {
                    cout << dist[u] << endl;
                    return 0;
                }
            }
        }
    }
    cout << "impossible" << endl;
    return 0;
}
