import java.util.*;
import java.io.*;
import java.math.*;

public class jeroen_bmatch
{
	public static void main(String[] args) throws Exception
	{
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		String[] ps = in.readLine().split(" ");
		int n = Integer.valueOf(ps[0]);
		int k = Integer.valueOf(ps[1]);
		String[] orders = in.readLine().split(" ");
		String[] pinned = in.readLine().split(" ");
		ps = in.readLine().split(" ");
		char ct = ps[0].charAt(0);
		char cm = ps[1].charAt(0);

		// Make a bipartite matching between tickets and orders
		// from the computer
		MaxBipartiteMatch mf = new MaxBipartiteMatch(n, k);
		boolean[] exclude = new boolean[127];
		for(int i = 0; i < n; i++) {
			// Exclude orders that would disprove the claim
			if(orders[i].charAt(0) != cm && orders[i].charAt(1) == ct) {
				exclude[orders[i].charAt(0)] = true;
				exclude[orders[i].charAt(1)] = true;
				continue;
			}

			for(int j = 0; j < k; j++) {
				char p = pinned[j].charAt(0);
				if(orders[i].charAt(0) == p || orders[i].charAt(1) == p)
					mf.add(i, j);
			}
		}

		// If we can't find a matching, the statement must be false
		if(k != mf.matching()) {
			System.out.println("false");
			return;
		}

		// All pinned up items for excluded cards need to be flipped
		ArrayList<Integer> toflip = new ArrayList<Integer>();
		for(int i = 0; i < k; i++)
			if(exclude[pinned[i].charAt(0)])
				toflip.add(i+1);

		// If there is nothing to flip, the claim must be true
		if(toflip.isEmpty()) {
			System.out.println("true");
			return;
		}

		// Print the answer
		System.out.println(toflip.size());
		for(int i : toflip)
			System.out.print(i + " ");
		System.out.println();
	}
	
	static class MaxBipartiteMatch
	{
		private int U, V;
		private int NIL;
		private LinkedList<Integer>[] adj;
		public MaxBipartiteMatch(int U, int V)
		{
			this.U = U;
			this.V = V;
			this.NIL = U + V;
			adj = new LinkedList[U];
			for (int i = 0; i < U; i++)
				adj[i] = new LinkedList<Integer>();
		}

		// 0 <= i < M, 0 <= j < N
		public void add(int i, int j)
		{
			adj[i].add(j);
		}

		private int[] Pair_U;
		private int[] Pair_V;
		private int[] Dist;

		private boolean BFS()
		{
			Dist = new int[U + V + 1];
			Queue<Integer> Q = new LinkedList<Integer>();
			for (int i = 0; i < U; i++) {
				if (Pair_U[i] == NIL) {
					Dist[i] = 0;
					Q.offer(i);
				} else
					Dist[i] = Integer.MAX_VALUE;
			}
			Dist[NIL] = Integer.MAX_VALUE;
			while (!Q.isEmpty()) {
				int u = Q.poll();
				if (Dist[u] < Dist[NIL]) {
					for (int v : adj[u]) {
						if (Dist[Pair_V[v]] == Integer.MAX_VALUE) {
							Dist[Pair_V[v]] = Dist[u] + 1;
							Q.offer(Pair_V[v]);
						}
					}
				}
			}
			return Dist[NIL] < Integer.MAX_VALUE;
		}
		
		private boolean DFS(int u)
		{
			if (u != NIL) {
				for (int v : adj[u]) {
					if (Dist[Pair_V[v]] == Dist[u] + 1) {
						if (DFS(Pair_V[v])) {
							Pair_V[v] = u;
							Pair_U[u] = v;
							return true;
						}
					}
				}
				Dist[u] = Integer.MAX_VALUE;
				return false;
			}
			return true;
			
		}
		
		public int matching()
		{
			Pair_U = new int[U];
			Pair_V = new int[V];
			for (int i = 0; i < U; i++)
				Pair_U[i] = NIL;
			for (int i = 0; i < V; i++)
				Pair_V[i] = NIL;
			int matching = 0;
			while (BFS())
				for (int i = 0; i < U; i++)
					if (Pair_U[i] == NIL)
						if (DFS(i))
							matching++;
			return matching;
		}
	}
}
