#!/usr/bin/env python3
from math import exp, log
from sys import setrecursionlimit
from typing import List


class SCC:
    def __init__(self, adj: List[List[int]]):
        self.n = len(adj)
        self.age = 0
        self.ncomps = 0
        self.comps = []
        self.adj = adj
        self.tidx = [0] * self.n
        self.lnk = [0] * self.n
        self.cnr = [0] * self.n
        self.st = []
        self.vis = [False] * self.n
        for i in range(self.n):
            if not self.tidx[i]:
                self.dfs(i)

    def dfs(self, v: int):
        self.tidx[v] = self.lnk[v] = self.age = self.age + 1
        self.vis[v] = True
        self.st.append(v)
        for w in self.adj[v]:
            if not self.tidx[w]:
                self.dfs(w)
                self.lnk[v] = min(self.lnk[v], self.lnk[w])
            elif self.vis[w]:
                self.lnk[v] = min(self.lnk[v], self.tidx[w])
        if self.lnk[v] != self.tidx[v]:
            return
        self.comps.append([])
        while True:
            w = self.st.pop()
            self.vis[w] = False
            self.cnr[w] = self.ncomps
            self.comps[-1].append(w)
            if w == v:
                break
        self.ncomps += 1


setrecursionlimit(100_000)

n, m = map(int, input().split())
P = list(map(float, input().split()))
adj = [[] for _ in range(n)]
for _ in range(m):
    u, v = map(lambda x: int(x) - 1, input().split())
    adj[u].append(v)

scc = SCC(adj)

C = [0] * n
R = [0.0] * scc.ncomps
sink = [True] * scc.ncomps
for i in range(scc.ncomps):
    for j in scc.comps[i]:
        C[j] = i
        R[i] += log(max(2 ** -1074, 1 - P[j]))

for i in range(n):
    for j in adj[i]:
        if C[i] != C[j]:
            sink[C[i]] = False

print(max(exp(R[i]) for i in range(scc.ncomps) if sink[i]))
