#!/usr/bin/env python3
import sys
from bisect import bisect_left
from functools import cache

# Idea: bottom-up dynamic programming, where
#     cost[i] := total cost of rides 0, ..., i-1; disregarding not-fully-used multi-ride tickets.
# Last day is special; it *does* include not-fully-used multi-ride tickets

# This solution uses binary search to look up the index of the last precious trip for a give
# date, for a total running time of O(n * k * log n); this is wasteful and can be replaced
# by a linear scan through the trip dates.

# Works in Python 3.10 (because it uses the key argument to bisect_right)


sys.setrecursionlimit(100000)


@cache
def prev(day):
    # Returns max{ i : t_i < t }
    # bisect_left returns the index of the next trip forward in time (or this exact trip if `day in trips`). So:
    # - use `day + 1` to make sure we get `<` rather than `<=`
    # - decrement the returned index, because we're looking for the index _before_ the next trip forward in time
    return bisect_left(trips, day + 1, key=lambda t: t[0]) - 1


@cache
def opt(i):
    if i < 0: return 0
    if k == 0: return trips[i][1] + opt(i - 1)  # pay regular fare
    return min(
        # pay regular fare: f_i + OPT(i - 1)
        trips[i][1] + opt(i - 1),
        # j-th pass expires today: min_j { c_j + OPT(max(i - d_j, prev(t_i - p_j))) }
        min(cost + opt(max(i - days, prev(trips[i][0] - period)))
            for j, (period, days, cost) in enumerate(passes))
    )


n, k = map(int, input().split())
trips = tuple(tuple(map(int, input().split())) for _ in range(n))
passes = tuple(tuple(map(int, input().split())) for _ in range(k))
print(opt(n - 1))
