#!/usr/bin/env python3
from bisect import bisect_right

# Idea: bottom-up dynamic programming, where
#     cost[i] := total cost of rides 0, ..., i-1; disregarding not-fully-used multi-ride tickets.
# Last day is special; it *does* include not-fully-used multi-ride tickets

# This solution uses binary search to look up the index of the last precious trip for a give
# date, for a total running time of O(n * k * log n); this is wasteful and can be replaced
# by a linear scan through the trip dates.

# Works in Python 3.10 (because it uses the key argument to bisect_right)


n, k = map(int, input().split())
trips = tuple(tuple(map(int, input().split())) for _ in range(n))
passes = tuple(tuple(map(int, input().split())) for _ in range(k))

# Initially the cost array "uses no discounts"
cost = [0] + [sum(fare for (_, fare) in trips)] * n

for i, (date, fare) in enumerate(trips):
    cost[i + 1] = min(cost[i + 1], cost[i] + fare)
    for days, rides, price in passes:
        j = min(i + rides, bisect_right(trips, date + days - 1, key=lambda t: t[0]))
        cost[j] = min(cost[j], cost[i] + price)
print(cost[n])
