#!/usr/bin/env python3
# @EXPECTED_RESULTS@: ACCEPTED, TIME_LIMIT_EXCEEDED

from collections import defaultdict, deque
from typing import List


def edmonds_karp(graph: List[dict[int, int]], source: int, sink: int) -> int:
    assert source != sink
    flow = 0

    while True:
        parent = [-1] * len(graph)
        parent[source] = 0
        queue = deque([source])

        while queue:
            x = queue.popleft()
            for y, cap in graph[x].items():
                if parent[y] == -1 and cap > 0:
                    parent[y] = x
                    queue.append(y)
                    if y == sink:
                        break       # warning: does not precisely transcribe `goto` from C++

        if parent[sink] == -1:
            return flow

        min_flow = int(1e9)
        v = sink
        while v != source:
            p = parent[v]
            min_flow = min(min_flow, graph[p][v])
            v = p

        flow += min_flow
        v = sink
        while v != source:
            p = parent[v]
            graph[p][v] -= min_flow
            if graph[p][v] <= 0:
                del graph[p][v]
            graph[v][p] += min_flow
            v = p


def permits_perfect_matching(g: List[List[int]], size_right: int) -> bool:
    size_left = len(g)
    source = size_left + size_right
    sink = source + 1

    network = [defaultdict(int) for _ in range(sink + 1)]
    for i in range(size_left):
        network[source][i] = 1
        for j in g[i]:
            network[i][j + size_left] = 1
    for j in range(size_right):
        network[j + size_left][sink] = 1

    return edmonds_karp(network, source, sink) == size_right


class Card:
    def __init__(self, letter, digit):
        self.letter = letter
        self.digit = digit

    def __iter__(self):
        return iter((self.letter, self.digit))


def print_vector(v):
    assert v
    print(len(v))
    print(' '.join(str(x + 1) for x in v))


n, k = map(int, input().split())
deck = [Card(*card) for card in input().split()]
table = input().split()
D, L = input().split()

legal, illegal = [], []
for i, (l, d) in enumerate(deck):
    if l != L and d == D:
        illegal.append(i)
    else:
        legal.append(i)

if not illegal:
    exit(print("true"))

graph = [[] for _ in range(n)]
for i in legal:
    l, d = deck[i]
    for j, c in enumerate(table):
        if l == c or d == c:
            graph[i].append(j)

if not permits_perfect_matching(graph, k):
    exit(print("false"))

check_letter = [False] * 26
for i in illegal:
    check_letter[ord(deck[i].letter) - ord('A')] = True
assert not check_letter[ord(L) - ord('A')]

flip = []
for j, c in enumerate(table):
    if c == D:
        flip.append(j)
    elif c >= 'A' and check_letter[ord(c) - ord('A')]:
        flip.append(j)

if not flip:
    print("true")
else:
    print_vector(flip)
