#include <iostream>
#include <algorithm>
#include <vector>
#include <climits>
#include <stack>
#include <functional>

#define rep(i, a, b) for (int i = a; i < b; ++i)

using namespace std;
using vi = vector<int>;
using vvi = vector<vi>;

vi next_larger(const vi& a, bool nx, auto&& c) {
    vi ans(size(a), nx ? ssize(a) : -1);
    stack<int> st;
    rep(j, 0, ssize(a)) {
        int i = nx ? j : ssize(a) - j - 1;
        while (!st.empty() && c(a[st.top()], a[i]))
            ans[st.top()] = i, st.pop();
        st.push(i);
    }
    return ans;
}

template <typename T>
auto transpose(const vector<T>& a) {
    vector b(size(a[0]), T(size(a), 0));
    rep(i, 0, size(a)) rep(j, 0, size(a[0])) b[j][i] = a[i][j];
    return b;
}

int main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);
    int w, h, q, qw, qh; cin >> h >> w >> q;

    vector<string> M(h);
    for (auto &row : M) cin >> row;

    bool transposed = h < w;
    if (transposed) swap(h, w), M = transpose(M);

    vvi ans(h+1, vi(w+1));

    vi c(w);
    rep(i, 0, h) {
        rep(j, 0, w) c[j] = M[i][j] == '#' ? 0 : c[j] + 1;
        vi next_smaller = next_larger(c, true, greater());
        rep(j, 0, w) {
            if (c[j] == 0) continue;
            for (int k = j; k < w && c[k] > 0; k = next_smaller[k]) {
                int nx = next_smaller[k];
                ans[c[k]][k-j]--;
                ans[c[k]][nx-j]++;
            }
        }
    }

    rep(i, 1, h+1) for (int j = w-1; j >= 0; --j) ans[i][j] += ans[i][j+1];
    rep(j, 1, w+1) for (int i = h-1; i >= 0; --i) ans[i][j] += ans[i+1][j];
    rep(i, 0, q) {
        cin >> qh >> qw;
        cout << (transposed ? ans[qw][qh] : ans[qh][qw]) << "\n";
    }
}
