#include <iostream>
#include <vector>
#include <x86intrin.h>
#include <cstdint>
#include <cstring>

#pragma GCC optimize("O3")
#pragma GCC target("avx2,fma,sse,sse2")

// http://0x80.pl/articles/simd-strfind.html#sse-avx2
int32_t avx2_strstr_anysize(const char *s, size_t n, const char *needle, size_t k) {
    if (n < k) return 0;
    int32_t matches = 0;
    int32_t prev_match = -n-1;
    while (((uintptr_t)s) % 32 != 0 && n > 0) {
        if (0 - prev_match >= k && memcmp(s, needle, k) == 0) {
            ++matches;
            prev_match = 0;
        }
        ++s;
        --n;
        --prev_match;
    }
    const __m256i first = _mm256_set1_epi8(needle[0]);
    const __m256i last = _mm256_set1_epi8(needle[k - 1]);
    for (size_t i = 0; i < n; i += 32) {
        if (i + 31 - prev_match < k) continue;
        const __m256i block_first = _mm256_loadu_si256(reinterpret_cast<const __m256i *>(s + i));
        const __m256i block_last = _mm256_loadu_si256(reinterpret_cast<const __m256i *>(s + i + k - 1));
        const __m256i eq_first = _mm256_cmpeq_epi8(first, block_first);
        const __m256i eq_last = _mm256_cmpeq_epi8(last, block_last);
        for (uint32_t mask = _mm256_movemask_epi8(_mm256_and_si256(eq_first, eq_last)); mask != 0; mask &= mask - 1) {
            const auto bitpos = __builtin_ctz(mask);
            if (i + bitpos - prev_match < k) continue;
            if (memcmp(s + i + bitpos, needle, k) == 0) {
                ++matches;
                prev_match = i + bitpos;
            }
        }
    }
    return matches;
}

#define rep(i, a, b) for (int i = a; i < b; i++)
#define sz(x) (int)(x).size()

using namespace std;

using vi = vector<int>;

int main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);
    string s;
    cin >> s;
    int n = sz(s);
    int best = 1 + sz(s);
    rep(i, 0, sz(s)) {
        int prev_overlap = n;
        rep(j, i + 2, n + 1) {
            int w = j - i;
            if ((n - i) + n - ((n - i) - 1) * prev_overlap >= best) break;
            int non_overlapping = 1;
            non_overlapping += avx2_strstr_anysize(s.data() + i + w, n - (i + w), s.data() + i, w);
            best = min(best, w + n - (w - 1) * non_overlapping);
            prev_overlap = non_overlapping;
        }
    }
    cout << best << endl;
}
