#include <array>
#include <cassert>
#include <iostream>
#include <vector>

#define all(x) begin(x), end(x)
#define sz(x) (int((x).size()))
#define rep(i, a, b) for (int i = (a); i < (b); ++i)

using vi = std::vector<int>;
using vvi = std::vector<vi>;

struct card {
    char letter, digit;
};

void print(const vi &v) {
    assert(!v.empty());
    std::cout << v.size() << '\n';
    for (int i = 0; i < sz(v); i++) {
        if (i) std::cout << ' ';
        std::cout << v[i] + 1;
    }
    std::cout << std::endl;
}

int main() {
    int n, k;  // no. cards in deck and on table, respectively
    std::cin >> n >> k;

    std::vector<card> deck(n);
    for (card &c : deck) std::cin >> c.letter >> c.digit;

    std::vector<char> table;
    for (int j = 0; j < k; j++) {
        char c;
        std::cin >> c;
        table.push_back(c);
    }

    char D, L;           // claim: "Each card on the table with a {D} on
    std::cin >> D >> L;  //         one side has an {L} on the other side."

    // the claim is essentially  D => L  ≡  ¬D ∨ L
    // so the "illegal" cards are those with  D ∧ ¬L
    vi legal, illegal;
    for (int i = 0; i < n; i++) {
        auto [l, d] = deck[i];
        (l != L && d == D ? illegal : legal).push_back(i);
    }

    if (illegal.empty()) {
        std::cout << "true" << std::endl;
        return 0;
    }

    // Error: in the following, we check for each table card independently whether it must be
    // illegal. However, this is only sufficient if each deck card is available in unlimited
    // quantities. Otherwise, the limited supply imposes additional constraints.
    for (char c : table) {
        if (c == D) {
            // If all cards in the deck with a {D} on one side do *not* have an {L} on the other
            // side, then the claim is certainly false.
            bool all = true;
            for (auto [l, d] : deck) all &= l != L || d != D;
            if (all) {
                std::cout << "false" << std::endl;
                return 0;
            }
        }
        if (c >= 'A' && c != L) {
            // If all cards in the deck with a {c} on one side have a {D} on the other side, then
            // the claim is certainly false.
            bool all = true;
            for (auto [l, d] : deck) all &= l != c || d == D;
            if (all) {
                std::cout << "false" << std::endl;
                return 0;
            }
        }
    }

    // Now, the claim may be either true or false: a legal matching exists, but
    // any legal card on the table could actually be an illegal one, as long as
    // the upright side is the same. Hence, we now determine which cards we need
    // to flip to (dis)prove the claim.

    std::array<bool, 26> checkLetter;
    checkLetter.fill(false);
    for (int i : illegal) checkLetter[deck[i].letter - 'A'] = true;
    assert(!checkLetter[L - 'A']);

    vi flip;
    for (int j = 0; j < k; j++) {
        char c = table[j];

        // Since we know the deck contains illegal cards, we must flip all cards
        // with a {D}.
        if (c == D) flip.push_back(j);

        // If a table card shows a letter `l` and there exists an illegal card
        // with that letter, we must flip it. Note that we never have to flip
        // cards with `l = L`.
        if (c >= 'A' && checkLetter[c - 'A']) flip.push_back(j);
    }

    if (flip.empty())
        // The table contains no illegal cards (since none of the visible sides
        // match). Therefore, all existing matchings are legal.
        std::cout << "true" << std::endl;
    else
        print(flip);
}
