#include <array>
#include <cassert>
#include <iostream>
#include <vector>

#define all(x) begin(x), end(x)
#define sz(x) (int((x).size()))
#define rep(i, a, b) for (int i = (a); i < (b); ++i)

using vi = std::vector<int>;
using vvi = std::vector<vi>;

bool dfs(int a, int L, const vvi &g, vi &btoa, vi &A, vi &B) {
    if (A[a] != L) return 0;
    A[a] = -1;
    for (int b : g[a]) if (B[b] == L+1) {
        B[b] = 0;
        if (btoa[b] == -1 || dfs(btoa[b], L+1, g, btoa, A, B))
            return btoa[b] = a, 1;
    }
    return 0;
}

// https://github.com/kth-competitive-programming/kactl/blob/main/content/graph/hopcroftKarp.h
// licensed under CC0  :)
int hopcroftKarp(const vvi &g, vi &btoa) {
    int res = 0;
    vi A(g.size()), B(btoa.size()), cur, next;
    for (;;) {
        fill(all(A), 0);
        fill(all(B), 0);
        /// Find the starting nodes for BFS (i.e. layer 0).
        cur.clear();
        for (int a : btoa) if(a != -1) A[a] = -1;
        rep(a,0,sz(g)) if(A[a] == 0) cur.push_back(a);
        /// Find all layers using bfs.
        for (int lay = 1;; lay++) {
            bool islast = 0;
            next.clear();
            for (int a : cur) for (int b : g[a]) {
                if (btoa[b] == -1) {
                    B[b] = lay;
                    islast = 1;
                }
                else if (btoa[b] != a && !B[b]) {
                    B[b] = lay;
                    next.push_back(btoa[b]);
                }
            }
            if (islast) break;
            if (next.empty()) return res;
            for (int a : next) A[a] = lay;
            cur.swap(next);
        }
        /// Use DFS to scan for augmenting paths.
        rep(a,0,sz(g))
            res += dfs(a, 0, g, btoa, A, B);
    }
}

bool permitsPerfectMatching(const vvi &g, int sizeRight) {
    vi btoa(sizeRight, -1);
    return hopcroftKarp(g, btoa) == sizeRight;
}

struct card {
    char letter, digit;
};

void print(const vi &v) {
    std::cout << v.size() << '\n';
    for (int i = 0; i < sz(v); i++) {
        if (i) std::cout << ' ';
        std::cout << v[i] + 1;
    }
    std::cout << std::endl;
}

int main() {
    int n, k;  // no. cards in deck and on table, respectively
    std::cin >> n >> k;

    std::vector<card> deck(n);
    for (card &c : deck) std::cin >> c.letter >> c.digit;

    std::vector<char> table;
    for (int j = 0; j < k; j++) {
        char c;
        std::cin >> c;
        table.push_back(c);
    }

    char D, L;           // claim: "Each card on the table with a {D} on
    std::cin >> D >> L;  //         one side has an {L} on the other side."

    // the claim is essentially  D => L  ≡  ¬D ∨ L
    // so the "illegal" cards are those with  D ∧ ¬L
    vi legal, illegal;
    for (int i = 0; i < n; i++) {
        auto [l, d] = deck[i];
        (l != L && d == D ? illegal : legal).push_back(i);
    }

    if (illegal.empty()) {
        std::cout << "true" << std::endl;
        return 0;
    }

    // If, considering only legal cards, we cannot find a matching between cards
    // in the deck and on the table, then the table must contain an illegal card,
    // so the claim is certainly false.
    {
        vvi g(deck.size());
        for (int i : legal) {
            auto [l, d] = deck[i];
            for (int j = 0; j < k; j++)
                if (l == table[j] || d == table[j])
                    g[i].push_back(j);
        }

        if (!permitsPerfectMatching(g, table.size())) {
            std::cout << "false" << std::endl;
            return 0;
        }
    }

    // Now, the claim may be either true or false: a legal matching exists, but
    // any legal card on the table could actually be an illegal one, as long as
    // the upright side is the same. Hence, we now determine which cards we need
    // to flip to (dis)prove the claim.

    std::array<bool, 26> checkLetter;
    checkLetter.fill(false);
    for (int i : illegal) checkLetter[deck[i].letter - 'A'] = true;
    assert(!checkLetter[L - 'A']);

    vi flip;
    for (int j = 0; j < k; j++) {
        char c = table[j];

        // Since we know the deck contains illegal cards, we must flip all cards
        // with a {D}.
        if (c == D) flip.push_back(j);

        // If a table card shows a letter `l` and there exists an illegal card
        // with that letter, we must flip it. Note that we never have to flip
        // cards with `l = L`.
        if (c >= 'A' && checkLetter[c - 'A']) flip.push_back(j);
    }

    print(flip);
}
