
#include <iostream>
#include <string>
#include <vector>
#include <queue>

using namespace std;

using ll = long long;
using ld = long double;
using pll = pair<ll, ll>;
using vll = vector<ll>;
using vld = vector<ld>;
using vpll = vector<pll>;
using vvll = vector<vll>;

#define INF 1000000000

template <typename N = ll>
struct flow_edge{
    ll j, r; N c, f; // to, reverse, capacity, flow
    flow_edge(ll j_, ll r_, N c_) : j(j_), r(r_), c(c_), f(0) {}
};

template <typename N = ll>
struct flow_graph : vector<vector<flow_edge<N>>>{
	flow_graph(ll n = 0) : vector<vector<flow_edge<N>>>(n) {}
	void add_arc(ll i, ll j, N c = 1){
	    auto &t = *this;
		t[i].emplace_back(j, t[j].size(), c);
		t[j].emplace_back(i, t[i].size() - 1, 0);
	}
};

// Find the maximum flow in a flow graph from s to t
template <typename N = ll>
struct dinic{
    flow_graph<N> &g;
    ll s, t; N flow;
    vll level;
    vector<typename vector<flow_edge<N>>::iterator> its;

    dinic(flow_graph<N> &g_, ll s_, ll t_) : g(g_), s(s_), t(t_), flow(0), level(g.size()), its(g.size()) {}

    N augment(ll i, N x){
        if (i == t) return x;
        N s = 0;
        for (; its[i] != g[i].end(); ++its[i]){
            flow_edge<N> &e = *its[i];
            if (e.f == e.c || level[e.j] != level[i] + 1) continue;
            N aug = augment(e.j, min(x, e.c - e.f));
            if (aug == 0) continue;
            e.f += aug; g[e.j][e.r].f -= aug; s += aug; x -= aug;
            if (x == 0) break;
        }
        return s;
    }

    dinic &run(){
        while (true){
            fill(level.begin(), level.end(), -1); level[s] = 0;
            queue<ll> todo; todo.push(s);
            while (!todo.empty()){
                ll i = todo.front(); todo.pop();
                its[i] = g[i].begin();
                for (flow_edge<N> e : g[i]) if (e.f < e.c && level[e.j] == -1) {level[e.j] = level[i] + 1; todo.push(e.j);}
            }
            if (level[t] == -1) return *this;
            flow += augment(s, INF);
        }
    }
};

int main(){
    ll n, k;
    cin >> n >> k;
    vector<string> cards(n);
    for (ll i = 0; i < n; ++i) cin >> cards[i];
    vll pins(k);
    for (ll i = 0; i < k; ++i){
        string pin;
        cin >> pin;
        pins[i] = pin[0] - 'A';
        if (pins[i] < 0 || pins[i] >= 26) pins[i] = 26 + (pin[0] - '0');
    }
    string from, to;
    cin >> from >> to;
    
    flow_graph<> g(n + 38); // n cards, 26+10 symbols, s&t
    vector<bool> problem(36);
    for (ll i = 0; i < 36; ++i) g.add_arc(n + i, n + 37);
    for (ll i = 0; i < n; ++i){
        if (cards[i][1] != from[0] || cards[i][0] == to[0]){
            g.add_arc(n + 36, i);
            g.add_arc(i, n + (cards[i][0] - 'A'));
            g.add_arc(i, n + 26 + (cards[i][1] - '0'));
        }
        else problem[26 + (from[0] - '0')] = problem[cards[i][0] - 'A'] = true;
    }
    
    dinic solver(g, n + 36, n + 37);
    if (solver.run().flow < k) cout << "false" << endl;
    else{
        vll flips;
        for (ll i = 0; i < k; ++i) if (problem[pins[i]]) flips.push_back(i + 1);
        if (flips.empty()) cout << "true" << endl;
        else{
            cout << flips.size() << endl;
            cout << flips[0];
            for (ll i = 1; i < flips.size(); ++i) cout << ' ' << flips[i];
            cout << endl;
        }
    }
    return 0;
}
