#include <cmath>
#include <iomanip>
#include <iostream>
#include <queue>
#include <vector>
using namespace std;

typedef long long ll;
typedef long double ld;
typedef pair<ll, ll> ii;
typedef vector<ll> vi;
typedef vector<vi> vvi;
typedef vector<ii> vii;

#define x first
#define y second
#define pb push_back
#define eb emplace_back
#define all(v) (v).begin(), (v).end()

struct SCC {
    int n, age = 0, ncomps = 0;
    vvi adj, comps;
    vi tidx, lnk, cnr, st;
    vector<bool> vis;
    SCC(vvi& _adj) : n(_adj.size()), adj(_adj), tidx(n, 0), lnk(n), cnr(n), vis(n, false) {
        for(ll i = 0; i < n; i++) {
            if(!tidx[i]) dfs(i);
        }
    }

    void dfs(int v) {
        tidx[v] = lnk[v] = ++age;
        vis[v]           = true;
        st.pb(v);
        for(int w : adj[v]) {
            if(!tidx[w])
                dfs(w), lnk[v] = min(lnk[v], lnk[w]);
            else if(vis[w])
                lnk[v] = min(lnk[v], tidx[w]);
        }
        if(lnk[v] != tidx[v]) return;
        comps.pb(vi());
        int w;
        do {
            vis[w = st.back()] = false;
            cnr[w]             = ncomps;
            comps.back().pb(w);
            st.pop_back();
        } while(w != v);
        ncomps++;
    }
};

void run() {
    ll n, m;
    cin >> n >> m;
    vector<ld> P(n);
    for(ll i = 0; i < n; i++) {
        cin >> P[i];
    }
    vvi adj(n);
    for(ll i = 0; i < m; i++) {
        ll u, v;
        cin >> u >> v;
        adj[u - 1].pb(v - 1);
    }
    SCC S(adj);

    vi C(n);
    vector<ld> R(S.ncomps, 0);
    vector<bool> sink(S.ncomps, true);
    for(ll i = 0; i < S.ncomps; i++) {
        for(ll j : S.comps[i]) {
            C[j] = i;
            R[i] += log(1 - P[j]);
        }
    }
    for(ll i = 0; i < n; i++) {
        for(ll j : adj[i]) {
            if(C[i] != C[j]) {
                sink[C[i]] = false;
            }
        }
    }
    ld res = 0;
    for(ll i = 0; i < S.ncomps; i++) {
        if(sink[i]) {
            res = max(res, exp(R[i]));
        }
    }
    cout << res << endl;
}

signed main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout << fixed << setprecision(20); // You should NOT use `fixed`
    run();
    return 0;
}
