#include <iostream>
#include <vector>
#include <random>
#include <chrono>
#include <numeric>
#include <cassert>

#define rep(i, a, b) for (int i = (a); i < (b); i++)
#define all(x) (x).begin(), (x).end()
#define sz(x) (int)ssize(x)
using namespace std;
using ll = long long;
using pii = pair<ll, ll>;


/**
 * Author: Johan Sannemo, pajenegod
 * Date: 2015-02-06
 * License: CC0
 * Source: Folklore
 * Description: Range Minimum Queries on an array. Returns
 * min(V[a], V[a + 1], ... V[b - 1]) in constant time.
 * Usage:
 *  RMQ rmq(values);
 *  rmq.query(inclusive, exclusive);
 * Time: $O(|V| \log |V| + Q)$
 * Status: stress-tested
 */

template<class T>
struct RMQ {
    vector<vector<T>> jmp;
    RMQ(const vector<T>& V) : jmp(1, V) {
        for (int pw = 1, k = 1; pw * 2 <= sz(V); pw *= 2, ++k) {
            jmp.emplace_back(sz(V) - pw * 2 + 1);
            rep(j,0,sz(jmp[k]))
                jmp[k][j] = min(jmp[k - 1][j], jmp[k - 1][j + pw]);
        }
    }
    T query(int a, int b) {
        assert(a < b); // or return inf if a == b
        int dep = 31 - __builtin_clz(b - a);
        return min(jmp[dep][a], jmp[dep][b - (1 << dep)]);
    }
};

int main() {
    cin.tie(0)->sync_with_stdio(0);
    cin.exceptions(cin.failbit);

    int n, m; cin >> n >> m;

    vector<pii> p(n);
    rep(i, 0, n) {
        cin >> p[i].first;
        p[i].second = i;
    }
    vector used(n, false);
    RMQ<pii> rmq(p);
    vector<pii> intervals(m), relevant;
    for (auto& [l, r] : intervals) {
        cin >> l >> r;
        --l;
        auto [c, x] = rmq.query(l, r);
        if (used[x]) continue;
        used[x] = true;
        relevant.emplace_back(-x, c);
    }
    sort(all(relevant));
    sort(all(intervals), [](auto a, auto b) { return a.second < b.second; });
    ll covered = -1, ans = 0;
    for (auto [l, r] : intervals) {
        if (covered >= l) continue;
        while (sz(relevant) >= 2 && -end(relevant)[-2].first < r) relevant.pop_back();
        auto [x, c] = relevant.back();
        relevant.pop_back();
        covered = -x;
        ans += c;
    }

    cout << ans << '\n';

    return 0;
}