#include <iostream>
#include <vector>
#include <functional>
#include <algorithm>
#include <numeric>
#include <random>
#include <chrono>
#include <ranges>

#pragma GCC optimize("Ofast")
#pragma GCC target("avx,avx2,fma")

#define rep(i, n) for (int i = 0; i < (n); ++i)
#define all(x) begin(x), end(x)

using namespace std;
using ll = long long;

struct avg {
    ll sum, cnt;

    avg() : sum(0), cnt(0) {}

    avg(ll sum) : sum(sum), cnt(1) {}

    avg(ll sum, ll cnt) : sum(sum), cnt(cnt) {}

    auto operator<=>(const avg &o) const {
        return sum * o.cnt <=> cnt * o.sum;
    }

    auto operator+(const avg &o) const {
        return avg{sum + o.sum, cnt + o.cnt};
    }

    auto operator+=(const avg &o) {
        return *this = *this + o;
    }

    auto operator-(const avg &o) const {
        return avg{sum - o.sum, cnt - o.cnt};
    }

    auto operator-=(const avg &o) {
        return *this = *this - o;
    }
};


int main() {
    auto start = chrono::high_resolution_clock::now();

    cin.tie(0)->sync_with_stdio(0);

    auto rng = std::mt19937(std::chrono::steady_clock::now().time_since_epoch().count());
    auto U = std::uniform_real_distribution<double>(0, 1);

    int n;
    cin >> n;
    vector<ll> a(n);
    for (auto &x: a) cin >> x;

    avg target{reduce(all(a)), n};

    // C++ 23 will be great...
    auto _below = std::ranges::filter_view(std::ranges::iota_view(0, n), [&](ll i) { return a[i] <= target; });
    vector<int> below(all(_below));
    shuffle(all(below), rng);

    const double TL = 1;
    const double MARGIN = 0.02;
    for (int sample = 0;
         TL - chrono::duration<double>(chrono::high_resolution_clock::now() - start).count() > MARGIN; sample++) {
        int i = below[sample % below.size()];
        avg cur = a[i];
        int l = i, r = i;
        vector<int> sol = {i};
        while (1) {
            bool can_take_left = l - 1 >= 0 && cur + a[l - 1] <= target;
            bool can_take_right = r + 1 < n && cur + a[r + 1] <= target;
            if (!can_take_left && !can_take_right) {
                if (l == 0 && r == n - 1) {
                    rep(j, n) cout << sol[j] + 1 << " \n"[j == n - 1];
                    return 0;
                }
                break;
            }
            if (can_take_left && can_take_right) {
                if (U(rng) < a[l - 1] / double(a[l - 1] + a[r + 1])) can_take_left = false;
                else can_take_right = false;
            }
            if (can_take_left) sol.push_back(--l), cur += a[l];
            else sol.push_back(++r), cur += a[r];
        }
    }
    cout << "impossible\n";
}
