#include <iostream>
#include <vector>
#include <functional>
#include <algorithm>
#include <numeric>
#include <random>
#include <chrono>
#include <ranges>

#define rep(i, n) for (int i = 0; i < (n); ++i)
#define all(x) begin(x), end(x)

using namespace std;
using ll = long long;

struct avg {
    ll sum, cnt;

    avg() : sum(0), cnt(0) {}

    avg(ll sum) : sum(sum), cnt(1) {}

    avg(ll sum, ll cnt) : sum(sum), cnt(cnt) {}

    auto operator<=>(const avg &o) const {
        return sum * o.cnt <=> cnt * o.sum;
    }

    auto operator+(const avg &o) const {
        return avg{sum + o.sum, cnt + o.cnt};
    }

    auto operator+=(const avg &o) {
        return *this = *this + o;
    }

    auto operator-(const avg &o) const {
        return avg{sum - o.sum, cnt - o.cnt};
    }

    auto operator-=(const avg &o) {
        return *this = *this - o;
    }
};


int main() {
    auto start = chrono::high_resolution_clock::now();

    cin.tie(0)->sync_with_stdio(0);

    auto rng = std::mt19937(std::chrono::steady_clock::now().time_since_epoch().count());

    int n;
    cin >> n;
    vector<ll> a(n);
    for (auto &x: a) cin >> x;

    avg target{reduce(all(a)), n};

    const double TL = 1;
    const double MARGIN = 0.02;
    while (TL - chrono::duration<double>(chrono::high_resolution_clock::now() - start).count() > MARGIN) {
        vector<int> sol;
        int l = 0, r = n - 1;
        avg cur;
        while (1) {
            if (l > r) {
                std::ranges::reverse(sol);
                rep(i, n) cout << sol[i] + 1 << " \n"[i == n - 1];
                return 0;
            }
            bool can_take_left = cur + a[l] >= target;
            bool can_take_right = cur + a[r] >= target;
            if (!can_take_left && !can_take_right)
                break;
            if (can_take_left && can_take_right) {
                if (rng() % 2) can_take_left = false;
                else can_take_right = false;
            }
            if (can_take_left) sol.push_back(l), cur += a[l++];
            else sol.push_back(r), cur += a[r--];
        }
    }
    cout << "impossible\n";
}
