#include <algorithm>
#include <climits>
#include <iostream>
#include <numeric>
#include <vector>
#define rep(i, b) for (int i = 0; i < (b); ++i)
#define all(x) begin(x), end(x)
using namespace std;

using ll = long long;
using pii = pair<int, int>;

struct Tree {
    typedef ll T;
    static constexpr T unit = LLONG_MAX;
    T f(T a, T b) { return min(a, b); }
    vector<T> s; int n;
    Tree(int n = 0, T def = unit) : s(2*n, def), n(n) {}
    void update(int pos, T val) {
        for (s[pos += n] = val; pos /= 2;)
            s[pos] = f(s[pos * 2], s[pos * 2 + 1]);
    }
    T query(int b, int e) {
        T ra = unit, rb = unit;
        for (b += n, e += n; b < e; b /= 2, e /= 2) {
            if (b % 2) ra = f(ra, s[b++]);
            if (e % 2) rb = f(s[--e], rb);
        }
        return f(ra, rb);
    }
};

int main() {
    cin.tie(0)->sync_with_stdio(0);

    int m,n; cin >> m >> n;
    vector<ll> a(m);
    rep(i, m) cin >> a[i];
    a.emplace_back(0);
    vector<pii> I(n);
    rep(i, n) cin >> I[i].first >> I[i].second;
    for (auto& [l, r] : I) l--;
    sort(all(I));

    vector<pii> monotone;
    for (auto [l, r] : I) {
        while (!monotone.empty() && monotone.back().second >= r)
            monotone.pop_back();
        monotone.emplace_back(l, r);
    }

    vector<int> finished(m+1);
    for (auto [l, r] : monotone) finished[r]++;
    partial_sum(all(finished), begin(finished));
    Tree dp(m+1);
    rep(i, m) {
        if (finished[i] == 0) {
            dp.update(i, a[i]);
            continue;
        }
        int x = monotone[finished[i]-1].first;
        dp.update(i, dp.query(x, i) + a[i]);
    }

    cout << dp.query(m-1, m) << '\n';

    return 0;
}
